<?php

namespace UnicaenDbImport\Domain;

use Assert\AssertionChain;
use Assert\InvalidArgumentException;
use Doctrine\DBAL\Connection;
use UnicaenDbImport\Config\ConfigException;
use Laminas\Stdlib\Parameters;
use UnicaenDbImport\Filter\ColumnValue\ColumnValueFilterInterface;

class Destination implements DestinationInterface
{
    const INTERMEDIATE_TABLE_NAME_PREFIX = 'TMP_';

    /**
     * @var Parameters
     */
    protected $config;

    /**
     * @var bool
     */
    protected $usesIntermediateTable = false;

    /**
     * @inheritDoc
     */
    public function setUsesIntermediateTable(bool $usesIntermediateTable = true)
    {
        $this->usesIntermediateTable = $usesIntermediateTable;
    }

    /**
     * @return bool
     */
    public function usesIntermediateTable(): bool
    {
        return $this->usesIntermediateTable;
    }

    /**
     * @param array $config
     * @return static
     * @throws \UnicaenDbImport\Config\ConfigException
     */
    static public function fromConfig(array $config): self
    {
        $inst = new static($config);

        try {
            $inst->validateConfig();
        } catch (InvalidArgumentException $e) {
            throw new ConfigException($e->getMessage());
        }

        return $inst;
    }

    protected function __construct(array $config)
    {
        $this->config = new Parameters($config);
    }

    /**
     * @return string
     */
    public function __toString(): string
    {
        return implode(' -- ', array_filter([
            $this->getTable(),
            $this->getWhere() ? ' WHERE ' . $this->getWhere() : null,
            $this->usesIntermediateTable ?
                'Via ' . $this->getIntermediateTable() . ' (auto_drop = ' . (int)$this->getIntermediateTableAutoDrop() . ')' :
                null,
        ]));
    }

    /**
     * @throws \Assert\InvalidArgumentException
     * @throws \UnicaenDbImport\Config\ConfigException
     */
    protected function validateConfig()
    {
        (new AssertionChain($this->config->get($key = 'code'), "Une string non vide est requise pour la clé suivante: $key"))
            ->notEmpty()
            ->string();

        if ($table = $this->config->get($key = 'table')) {
            (new AssertionChain($table, "Une string non vide est requise pour la clé suivante: $key"))
                ->notEmpty()
                ->string();
        }

        if ($this->config->get($key = 'select')) {
            throw ConfigException::illegalKey('select');
        }

        if ($where = $this->config->get($key = 'where'))
            (new AssertionChain($where, "Une string non vide est requise pour la clé suivante: $key"))
                ->notEmpty()
                ->string();

        (new AssertionChain($this->config->get($key = 'connection'), "Une instance de " . Connection::class . " est requise pour la clé suivante: $key"))
            ->notEmpty()
            ->isInstanceOf(Connection::class);

        (new AssertionChain($this->config->get($key = 'source_code_column'), "Une string non vide est requise pour la clé suivante: $key"))
            ->notEmpty()
            ->string();

    }

    public function getCode(): string
    {
        return $this->config->get('code');
    }

    public function getTable(): string
    {
        return $this->config->get('table');
    }

    public function getWhere(): ?string
    {
        return $this->config->get('where');
    }

    public function getConnection(): Connection
    {
        return $this->config->get('connection');
    }

    public function getSourceCodeColumn(): string
    {
        return $this->config->get('source_code_column');
    }

    public function getColumns(): array
    {
        return $this->config->get('columns');
    }

    public function getColumnValueFilter(): ?ColumnValueFilterInterface
    {
        return $this->config->get('column_value_filter');
    }

    public function getIdColumnSequence()
    {
        return $this->config->get('id_sequence');
    }

    public function getIntermediateTable(): string
    {
        if ($intermediateTable = $this->config->get('intermediate_table')) {
            return $intermediateTable;
        }

        return static::INTERMEDIATE_TABLE_NAME_PREFIX . $this->getTable();
    }

    public function getIntermediateTableAutoDrop(): bool
    {
        return (bool) $this->config->get('intermediate_table_auto_drop', false);
    }

    public function getLogTable(): string
    {
        return $this->config->get('log_table', DestinationInterface::LOG_TABLE_DEFAULT_NAME);
    }
}